FUNCTION CONT_LEVEL,a,nlevels=nlevels
;+
; NAME:
;	CONT_LEVEL (Function)
; PURPOSE:
;	To make appropriate contour levels.
; CATEGORY:     ^^^^^^^^^^^
;	Array mamipulation.
; CALLING SEQUENCE:
;	LEVELS=CONT_LEVEL(A,[NLEVELS=10])
; INPUTS:
;     A = 2 dimensional array to display or 1 dimensional array.
; KEYWORD PARAMETERS:
;     NLEVELS = the number of levels (default is 15)
; EXAMPLE OF USAGE:
;     IDL> levels=cont_level(a,nlevels=5)  ;;a is array to be draw contour.
;     contour levels =               ;; execute function 'cont_level'
;           0.00000      20.0000      40.0000      60.0000      80.0000
;     IDL> contour,a,levels=levels   ;; draw contour.
; MODIFICATION HISTORY:
;     T.Matsumoto 1994,2,22
;     T.Matsumoto 1994.1.19 bug fixed and change algorithm.
;-
on_error,2
if n_elements(nlevels) eq 0 then nlevels=15
if nlevels eq 0 then goto,bad1
nlevels=fix(nlevels)
maxa=max(a)
mina=min(a)
delta=abs(maxa-mina)/(nlevels-1)
;if delta eq 0 then return,0
n=alog10(delta)
if n ge 0 then n=fix(n) else n=fix(n-1)
;;;;;; delta a = interval of contour lines( fix(alog10(d*))=n );;;;;;;
d1  =1.0 *10.0^(n+1)
d05 =0.5 *10.0^(n+1)
d025=0.25*10.0^(n+1)
d02 =0.2 *10.0^(n+1)
d01 =0.1 *10.0^(n+1)
;;;;;; define contour levels temporary with some margins.
level1  =findgen(fix((maxa-mina)/d1)+2)*d1    +fix(mina/d1)*d1
level05 =findgen(fix((maxa-mina)/d05)+2)*d05  +fix(mina/d05)*d05
level025=findgen(fix((maxa-mina)/d025)+2)*d025+fix(mina/d025)*d025
level02 =findgen(fix((maxa-mina)/d02)+2)*d02  +fix(mina/d02)*d02
level01 =findgen(fix((maxa-mina)/d01)+2)*d01  +fix(mina/d01)*d01
;;;;;; evaluate number of contour lines
elements0 =where((level1   ge mina) and (level1   le maxa))
elements1 =where((level05  ge mina) and (level05  le maxa))
elements2 =where((level025 ge mina) and (level025 le maxa))
elements3 =where((level02  ge mina) and (level02  le maxa))
elements4 =where((level01  ge mina) and (level01  le maxa))
;;;;; if elements* = -1 then 
sz0 =size(elements0)
sz1 =size(elements1)
sz2 =size(elements2)
sz3 =size(elements3)
sz4 =size(elements4)
szm=indgen(5)  ;;; szm(*) = number of contour line for each line style.
if sz0(0) eq 1 then szm(0) = sz0(1) else szm(0) =0
if sz1(0) eq 1 then szm(1) = sz1(1) else szm(1) =0
if sz2(0) eq 1 then szm(2) = sz2(1) else szm(2) =0
if sz3(0) eq 1 then szm(3) = sz3(1) else szm(3) =0
if sz4(0) eq 1 then szm(4) = sz4(1) else szm(4) =0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
res=indgen(5)
res(*)=abs(szm(*)-nlevels)       ;;; contour level ܿΤ
;print,res
;print,'level1',level1
;print,'level05',level05
;print,'level025',level025
;print,'level02',level02
;print,'level01',level01
case res(0)<res(1)<res(2)<res(3)<res(4) of
    res(0): levels=level1(elements0)
    res(1): levels=level05(elements1)
    res(2): levels=level025(elements2)
    res(3): levels=level02(elements3)
    res(4): levels=level01(elements4)
endcase
print,'contour levels ='
print,levels
print,'min =',mina,'    max =',maxa
bad1:
return,levels
end
