PRO rainbowf,top=top,bottom=bottom,begs=begs,value=value,$
             bgh=bgh,bgs=bgs,bgv=bgv,$
             th=th,ts=ts,tv=tv
;+
; NAME:
;     rainbow256f
; PURPOSE:
;     making color index
;     from white through blue cyan green yellow red magenda
; CATEGORY:
;     General graphics.
; CALLING SEQUENCE:
;     rainbowf
; INPUTS:
;     None.
; OPTIONAL INPUT PARAMETERS:
;     None.
; KEYWORD PARAMETERS:
;     top = maximun color index that will be specifyed.
;           The style is integer.
;           Default value is 199
;     bottom=minmum color index that will be specifyed.
;           The style is integer.
;           Default value is 0
;     begs = beginning piont of saturation in color table ( from 0.0 to 1.0 )
;           Default value is 0.5
;     value = value of color table ( from 0.0 to 1.0 )
; -- maximun color index (!d.table_size). default is black --
;     th = top(maximum color index)'s hue.( from 0 to 360 )
;     ts = top(maximum color index)'s saturation (from 0.0 to 1.0)
;     tv = top(maximum color index)'s value (from 0.0 to 1.0)
; -- bottom color index (0-bottom). default is white --
;     bgh = back ground's hue.( from 0 to 360 )
;     bgs = back ground's saturation (from 0.0 to 1.0)
;     bgv = back ground's value (from 0.0 to 1.0)
; OUTPUTS:
;     No explicit outputs.
; COMMON BLOCKS:
;     common colors, r_orig, g_orig, b_orig, r_curr, g_curr, b_curr
; SIDE EFFECTS:
;     The currently selected display is affected.
; RESTRICTIONS:
;     None that are obvious.
; PROCEDURE:
;     TVLCT
; MODIFICATION HISTORY:
;     T.Matsumoto 1993.11.24	
;     K.Sugimoto 2003.1.24
;-
common colors, r_orig, g_orig, b_orig, r_curr, g_curr, b_curr
on_error,2                      ;Return to caller if an error occurs
if not keyword_set(top) then top=!d.table_size-1
if not keyword_set(bottom) then bottom=0
if top ge !d.table_size then top=!d.table_size-1
if bottom lt 0 then bottom=0
if n_elements(begs) eq 0 then begs=1./6.
if n_elements(value) eq 0 then value=1.0
if begs gt 1.0 then begs=1.0
hs=fltarr(top-bottom+1)
ss=fltarr(top-bottom+1)
vs=fltarr(top-bottom+1)
unit=fltarr(!d.table_size) & unit(*)=1.0
zero=fltarr(!d.table_size) & zero(*)=0.0
;
m1=fix((top-bottom)*1.0/6.0+bottom)
m5=fix((top-bottom)*5.0/6.0+bottom)
;
;;;;;; top color index 
htop=fltarr(!d.table_size)
stop=fltarr(!d.table_size)
vtop=fltarr(!d.table_size)
if n_elements(th)*n_elements(ts)*n_elements(tv) ne 0 then begin
   htop(*)=th
   stop(*)=ts
   vtop(*)=tv
endif else begin
   htop=zero
   stop=zero
   vtop=zero
endelse
;;;;;; set saturation 
ns=fix((top-bottom)*begs > 1)
saturation=findgen(ns+1)/float(ns)
saturation=[saturation,unit(0:top-bottom-ns)]
;;;;;; color index
for i=bottom,m1 do begin
	hs(i)=240.
	ss(i)=saturation(i)
	vs(i)=value
endfor
;
for i=m1,m5 do begin
	hs(i)=-(float(i)-m1)/float(m5-m1)*240.0+240.0
	ss(i)=saturation(i)
	vs(i)=value
endfor
;
for i=m5,top-bottom do begin
	hs(i)=-(float(i)-m5)/float(top-m5)*60.0+360.0
	ss(i)=saturation(i)
	vs(i)=value
endfor
;;;;;; bottom + color index + top region
h=[zero(0:bottom),hs(1:top-bottom-1),htop(0:!d.table_size-top)]
s=[zero(0:bottom),ss(1:top-bottom-1),stop(0:!d.table_size-top)]
v=[unit(0:bottom),vs(1:top-bottom-1),vtop(0:!d.table_size-top)]
;;;;;; back ground color
if  n_elements(bgh)*n_elements(bgs)*n_elements(bgv) eq 0 then begin
h(0:bottom)=0.0 & s(0:bottom)=0.0 & v(0:bottom)=1.0
endif else begin
h(0:bottom)=bgh & s(0:bottom)=bgs & v(0:bottom)=bgv
endelse
tvlct,h,s,v,/hsv
tvlct,r_curr,g_curr,b_curr,/get
tvlct,r_orig,g_orig,b_orig,/get
return
end


