pro tvcn, a,b,xx,yy, ASPECT = aspect, $
          INTERP = interp, Levels=levels,Nobyte=nobyte,EXTRAP = extrap,$
          TITLE =title, XTITLE = xtitle, YTITLE = ytitle,$
          NLEVELS=nlevels,C_COLORS=c_colors,FOLLOW=follow,$
          xticks=xticks,yticks=yticks,xtickv=xtickv,ytickv=ytickv,$ 
          xtickname=xtickname,ytickname=ytickname,xminor=xminor,yminor=yminor,$
          TOBYTE=tobyte,c_kwd=c_kwd
;+
; NAME:
;	TVCN
; PURPOSE:
;	Overlay an image and a contour plot.
; CATEGORY:
;	General graphics.
; CALLING SEQUENCE:
;	TVCN, A, B, XX, YY
; INPUTS:
;    A = 2 dimensional array to display.(TV)
;    B = 2 dimensional array to display.(CONTOUR)
; OPTIONAL INPUT PARAMETERS:
;    XX = 1 dimensional array to display.
;    YY = 1 dimensional array to display.
; KEYWORD PARAMETERS:
;    ASPECT = Set to retain image's aspect ratio.   
;   /INTERP = Set to bi-linear interpolate if image is resampled.
;   /NOBYTE = Set if the data array A is NOT byte-scaled. If this
;             keyword is set, the array A is translated to a byte
;             which is ranged from 0 to !d.n_colors-1. The system
;             variable, !d.n_colors, is the number of colors that your
;             window device can used. Note that Openwindow can use 200
;             colors but PostScript can use 256 colors for pseud
;             colors.
;   /TOBYTE = Set if the data array A is NOT byte-scaled. If this
;             keyword is set, the array A is translated to a byte
;             which is ranged from 0 to 255. This keyword resembles
;             /NOBYTE, but /NOBYTE translates the array A to a byte
;             which is ranged from 0 to !d.n_colors-1. The data which
;             correspond to color index from !d.n_colors to 255 will
;             be shown in same color as color index !  d.n_colors-1
;   /EXTRAP = Set if contour is extrapolated to edge of plot data window
;   /FOLLOW
;    LEVELS = Set cotour levels
;    NLEVELS = Number of contour levels
;    TITLE = title, given as strings
;    XTITLE = xtitle, given as strings
;    YTITLE = ytitle, given as strings
;    C_COLORS = The color index used to draw each contour
;    XTICKS = Same as the keyword of 'contour'
;    YTICKS = Same as the keyword of 'contour'
;    XTICKV = Same as the keyword of 'contour'
;    YTICKV = Same as the keyword of 'contour'
;    XTICKNAME = Same as the keyword of 'contour'
;    YTICKNAME = Same as the keyword of 'contour'
;    XMINOR = Same as the keyword of 'contour'
;    YMINOR = Same as the keyword of 'contour'
;    C_KWD  = Other keywords for a contour plot, 
;             which must be set as a string.
;        ex) tvcn,a,a,x,y,c_kwd='c_label=[1,0,1,0]',level=[1,2,3,4]
; OUTPUTS:
;	No explicit outputs.
; COMMON BLOCKS:
;	none.
; SIDE EFFECTS:
;	The currently selected display is affected.
; RESTRICTIONS:
;	None that are obvious.
; PROCEDURE:
;	If the device has scalable pixels then the image is written over
;	the plot window.
; MODIFICATION HISTORY:
;	DMS, May, 1988.
;	Yokoyama,  ????
;	Yngwie J. Matsumoto 1993/8/**
;       T. Matsumoto 1995/4/3 ɸ(xx,yy)դˤʤäƤƤϣ
;       T. Matsumoto Mon Feb 26 16:18:15 JST 1996
;-
;;;;;;;; error checking ;;;;;;;;;
on_error,2                      ;Return to caller if an error occurs
sz = size(a)			;Size of image
if sz(0) lt 2 then message, 'Parameter not 2D'
if n_elements(xx) le 0 then xx=findgen(sz(1))
if n_elements(yy) le 0 then yy=findgen(sz(2))
if not keyword_set(title) then title=!p.title
if not keyword_set(xtitle) then xtitle=!x.title
if not keyword_set(ytitle) then ytitle=!y.title
if n_elements(xticks) eq 0 then xticks=!x.ticks
if n_elements(yticks) eq 0 then yticks=!y.ticks
if n_elements(xtickv) eq 0 then xtickv=!x.tickv
if n_elements(ytickv) eq 0 then ytickv=!y.tickv
if n_elements(xminor) eq 0 then xminor=!x.minor
if n_elements(yminor) eq 0 then yminor=!y.minor
if n_elements(xtickname) eq 0 then xtickname=!x.tickname
if n_elements(ytickname) eq 0 then ytickname=!y.tickname

                                ;set window used by contour
contour,[[0,0],[1,1]],/nodata, xstyle=4, ystyle = 4 ;Otameshi

px = !x.window * !d.x_vsize	;Get size of window in device units
py = !y.window * !d.y_vsize
;py=py-8.
swx = px(1)-px(0)		;Size in x in device units
swy = py(1)-py(0)		;Size in Y
six = float(sz(1))		;Image sizes
siy = float(sz(2))
aspr=abs(yy(1)-yy(0))/abs(xx(1)-xx(0))
aspi = six / siy /aspr		;Image aspect ratio
aspw = swx / swy		;Window aspect ratio
f = aspi / aspw			;Ratio of aspect ratios

;;;;;;;;;;;;;;;; default parameters ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
if n_elements(nlevels) eq 0 then nlevels=20
if n_elements(levels) eq 0 then begin
    mx=max(b)
    mn=min(b)
    levels=mn+(mx-mn)/nlevels*findgen(nlevels)
endif
;;;;;;;;;;;;;;;; plot color-plot ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
if (!d.flags and 1) ne 0 then begin ;Scalable pixels? (device PS)
    if keyword_set(aspect) then begin ;Retain aspect ratio?
                                ;Adjust window size
        if f ge 1.0 then swy = swy / f  else swx = swx * f 
    endif

    if keyword_set(nobyte) or keyword_set(tobyte) then begin
        if keyword_set(nobyte) then top=!d.n_colors-2 else top=256-2
        tv,bytscl(a,top=top)+1B $
          ,px(0),py(0),xsize = swx, ysize = swy, /device
    endif else begin
        tv,a,px(0),py(0),xsize = swx, ysize = swy, /device
    endelse

endif else begin                ;Not scalable pixel (device X)
    if keyword_set(aspect) then begin
        if f ge 1.0 then swy = swy / f  else swx = swx * f 
    endif                       ;aspect
    if keyword_set(nobyte) or keyword_set(tobyte) then begin
        if keyword_set(nobyte) then top=!d.n_colors-1 else top=256-1
        tv,poly_2d(bytscl(a,top=top),$ ;Have to resample image
                   [[0,0],[six/swx,0]], [[0,siy/swy],[0,0]],$
                   keyword_set(interp),swx,swy), $
          px(0),py(0)
    endif else begin
        tv,poly_2d(a,$          ;Have to resample image
                   [[0,0],[six/swx,0]], [[0,siy/swy],[0,0]],$
                   keyword_set(interp),swx,swy), $
          px(0),py(0)
    endelse
endelse                         ;scalable pixels

;;;;;;;;;;;;;;;; contour color index ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
mx = !d.n_colors-1		;Brightest color
if not keyword_set(c_colors) then begin 
    c_colors = fltarr(nlevels) & c_colors(*)=mx
    c_colors(abs(0.6*nlevels):*) = 0 ;some top line color are black
endif else begin 
    c_colors=c_colors
endelse
;if !d.name eq 'PS' then c_colors = mx - c_colors ;invert line colors for pstscrp
;;;;;;;;;;;;;;; extraploration contour ;;;;;;;;;;;;;;;;;;;;;;;;;;;;
if keyword_set(extrap) then  begin
    b1=[b(0,*),b,b(sz(1)-1,*)]  ;extrapolation in x-direction
    xx1=[2*xx(0)-xx(1),xx,2*xx(sz(1)-1)-xx(sz(1)-2)] ;extrap. x-axis
    b2=[[b1(*,0)],[b1],[b1(*,sz(1)-1)]] ;extrapolation in   y-direction
    yy1=[2*yy(0)-yy(1),yy,2*yy(sz(2)-1)-yy(sz(2)-2)] ;extrap.y-axis
endif else begin
    b2=b & xx1=xx & yy1=yy
endelse
;;;;;;;;;;;;;;;; plot cotour plot ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
if n_elements(c_kwd) eq 0 then begin
    c_kwd=' ' 
endif else begin
    c_kwd=',' + c_kwd
endelse

r=execute("contour,b2,xx1,yy1,/noerase,/xst,/yst,"+ $	;Do the contour
  "pos = [px(0),py(0), px(0)+swx,py(0)+swy],/dev,"+ $
  "levels=levels,c_color=c_colors,follow=follow,"+ $
  "title=title, xtitle=xtitle, ytitle = ytitle,"+ $
  "xrange=[xx(0)-0.5*(xx(1)-xx(0)),"+ $
          "xx(sz(1)-1)+0.5*(xx(sz(1)-1)-xx(sz(1)-2))],"+ $
  "yrange=[yy(0)-0.5*abs(yy(1)-yy(0)),"+ $
          "yy(sz(2)-1)+0.5*abs(yy(sz(2)-1)-yy(sz(2)-2))],"+ $
  "xticks=xticks,yticks=yticks,xtickv=xtickv,ytickv=ytickv,"+ $
  "xtickname=xtickname,ytickname=ytickname,xminor=xminor,yminor=yminor" + $
   c_kwd)


return
end

