pro tvcn2ve, a,b,c,d,e,xx,yy, ASPECT = aspect, $
            INTERP = interp,Scale=scale,levels=levels,Limit=limit,$
            Nobyte=nobyte,$
            ISKIP = iskip, JSKIP = jskip,EXTRAP = extrap,$
            I0SKIP = i0skip, J0SKIP = j0skip,$
            TITLE = title, XTITLE = xtitle, YTITLE = ytitle,$
            NLEVELS=nlevels,C_COLORS=c_colors,FOLLOW=follow,$
            F_COLOR=f_color,V_COLOR=v_color,sample=sample,$
            xticks=xticks,yticks=yticks,xtickv=xtickv,ytickv=ytickv,$
            xtickname=xtickname,ytickname=ytickname,$
            xminor=xminor,yminor=yminor,$
            TOBYTE=tobyte,c_kwd=c_kwd,$
            C2_COLORS=c2_colors,l2evels=l2evels
;+
; NAME:
;	TVCNVE
; PURPOSE:
;	Overlay an image, a contour plot, and a vector plot.
; CATEGORY:
;	General graphics.
; CALLING SEQUENCE:
;	TVCNVE, A, B, C, D, XX, YY
; INPUTS:
;    A = 2 dimensional array to display.
;    B = 2 dimensional array to display.
;    C = 2 dimensional array to display.
;    D = 2 dimensional array to display.
; KEYWORD PARAMETERS:
;   /ASPECT = set to retain image's aspect ratio.   
;   /INTERP = set to bi-linear interpolate if image is resampled.
;   /NOBYTE = Set if the data array A is NOT byte-scaled. If this
;             keyword is set, the array A is translated to a byte
;             which is ranged from 0 to !d.n_colors-1. The system
;             variable, !d.n_colors, is the number of colors that your
;             window device can used. Note that Openwindow can use 200
;             colors but PostScript can use 256 colors for pseud
;             colors.
;   /TOBYTE = Set if the data array A is NOT byte-scaled. If this
;             keyword is set, the array A is translated to a byte
;             which is ranged from 0 to 255. This keyword resembles
;             /NOBYTE, but /NOBYTE translates the array A to a byte
;             which is ranged from 0 to !d.n_colors-1. The data which
;             correspond to color index from !d.n_colors to 255 will
;             be shown in same color as color index !  d.n_colors-1
;   /EXTRAP = set if contour is extrapolated to edge of data plot window.
;   /FOLLOW 
;    LEVELS = set cotour levels
;    NLEVELS = Number of contour levels
;    SCALE = set scaling factor for the arrows
;    LIMIT = set limit for the minimum arrow size
;    ISKIP = set skip of ploting arrows in x-direction (in mesh unit)
;    JSKIP = set skip of ploting arrows in y-direction (in mesh unit)
;    I0SKIP = set skip of starting point of ploting arrows 
;		  in x-direction (in mesh unit)
;    J0SKIP = set skip of starting point of ploting arrows 
;		  in y-direction (in mesh unit)
;    TITLE = title, given as strings
;    XTITLE = xtitle
;    YTITLE = ytitle
;    C_COLORS = The color index used to draw each contour
;    V_COLOR = The color index used to draw arrows
;    F_COLOR = The color index used to draw flame
;    /SAMPLE = set to draw sample arrow
;    XTICKS = Same as the keyword of 'contour'
;    YTICKS = Same as the keyword of 'contour'
;    XTICKV = Same as the keyword of 'contour'
;    YTICKV = Same as the keyword of 'contour'
;    XTICKNAME = Same as the keyword of 'contour'
;    YTICKNAME = Same as the keyword of 'contour'
;    XMINOR = Same as the keyword of 'contour'
;    YMINOR = Same as the keyword of 'contour'
;    C_KWD  = Other keywords for a contour plot, 
;             which must be set as a string.
;        ex) tvcnve,a,a,va,vb,x,y,c_kwd='c_label=[1,0,1,0]',level=[1,2,3,4]
; OUTPUTS:
;	No explicit outputs.
; COMMON BLOCKS:
;	none.
; SIDE EFFECTS:
;	The currently selected display is affected.
; RESTRICTIONS:
;	None that are obvious.
; PROCEDURE:
;	If the device has scalable pixels then the image is written over
;	the plot window.
; MODIFICATION HISTORY:
;	DMS, May, 1988.
;	Yokoyama, ????
;	Yngwie J. Matsumoto 1993.8.*
;	Yngwie J. Matsumoto 1993.10.8
;       T. Matsumoto Mon Feb 26 16:18:15 JST 1996
;       T. Matsumoto Wed Feb  5 22:23:36 JST 1997
;-
;;;;;;;;;;; error checking ;;;;;;;;;;
on_error,2                      ;Return to caller if an error occurs
sz = size(a)			;Size of image
if sz(0) lt 2 then message, 'Parameter not 2D'
if n_elements(xx) le 0 then xx=findgen(sz(1))
if n_elements(yy) le 0 then yy=findgen(sz(2))
if not keyword_set(title) then title=''
if not keyword_set(xtitle) then xtitle= ''
if not keyword_set(ytitle) then ytiele=''
                                ;set window used by contour
contour,[[0,0],[1,1]],/nodata, xstyle=4, ystyle = 4

px = !x.window * !d.x_vsize	;Get size of window in device units
py = !y.window * !d.y_vsize
;py=py-8.
swx = px(1)-px(0)		;Size in x in device units
swy = py(1)-py(0)		;Size in Y
six = float(sz(1))		;Image sizes
siy = float(sz(2))
aspr=abs(yy(1)-yy(0))/abs(xx(1)-xx(0))
aspi = six / siy /aspr		;Image aspect ratio
aspw = swx / swy		;Window aspect ratio
f = aspi / aspw			;Ratio of aspect ratios

;;;;;;;;;;;;;;;; default parameters ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;if n_elements(scale) eq 0 then scale=0.5
if n_elements(limit) eq 0 then limit=0.
if n_elements(nlevels) eq 0 then nlevels=20
if n_elements(levels) eq 0 then begin
    mx=max(b)
    mn=min(b)
    levels=mn+(mx-mn)/nlevels*findgen(nlevels)
endif
;;;;;;;;;;;;;;;; plot color-plot ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
if (!d.flags and 1) ne 0 then begin ;Scalable pixels?
    if keyword_set(aspect) then begin ;Retain aspect ratio?
                                ;Adjust window size
        if f ge 1.0 then swy = swy / f else swx = swx * f
    endif

    if keyword_set(nobyte) or keyword_set(tobyte) then begin
        if keyword_set(nobyte) then top=!d.n_colors-2 else top=256-2
        tv,bytscl(a,top=top)+1B $
          ,px(0),py(0),xsize = swx, ysize = swy, /device
    endif else begin
        tv,a,px(0),py(0),xsize = swx, ysize = swy, /device
    endelse

endif else begin                ;Not scalable pixels	
    if keyword_set(aspect) then begin
        if f ge 1.0 then swy = swy / f else swx = swx * f
    endif                       ;aspect
    if keyword_set(nobyte) or keyword_set(tobyte) then begin
        if keyword_set(nobyte) then top=!d.n_colors-1 else top=256-1
        tv,poly_2d(bytscl(a,top=top),$ ;Have to resample image
                   [[0,0],[six/swx,0]], [[0,siy/swy],[0,0]],$
                   keyword_set(interp),swx,swy), $
          px(0),py(0)
    endif else begin
        tv,poly_2d(a,$          ;Have to resample image
                   [[0,0],[six/swx,0]], [[0,siy/swy],[0,0]],$
                   keyword_set(interp),swx,swy), $
          px(0),py(0)
    endelse
endelse                         ;scalable pixels

;;;;;;;;;;;;;;;; plot vector plot ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
vfield,c,d,xx,yy,/noerase,$
  pos = [px(0),py(0), px(0)+swx,py(0)+swy],/dev,scale=scale,limit=limit $
  ,is=iskip,js=jskip,margx=0.5*abs(xx(1)-xx(0)),margy=0.5*abs(yy(1)-yy(0)),$
  title=title,xtitle=xtitle,ytitle=ytitle,i0skip=i0skip,j0skip=j0skip,$
  sample=sample,v_color=v_color,f_color=f_color,$
  xticks=xticks,yticks=yticks,xtickv=xtickv,ytickv=ytickv,$
  xtickname=xtickname,ytickname=ytickname,$
  xminor=xminor,yminor=yminor

;;;;;;;;;;;;;;;; contour color index ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
mx = !d.n_colors-1		;Brightest color
if not keyword_set(c_colors) then begin 
    c_colors = fltarr(nlevels) & c_colors(*)=mx
    c_colors(abs(0.6*nlevels):*) = 0 ;some top line color are black
endif else begin 
    c_colors=c_colors
endelse
;if !d.name eq 'PS' then c_colors = mx - c_colors ;invert line colors for pstscrp
;;;;;;;;;;;;;;;; contour color index ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
mx = !d.n_colors-1		;Brightest color
if not keyword_set(c2_colors) then begin 
    c2_colors = fltarr(nlevels) & c2_colors(*)=mx
    c2_colors(abs(0.6*nlevels):*) = 0 ;some top line color are black
endif else begin 
    c2_colors=c2_colors
endelse
;if !d.name eq 'PS' then c_colors = mx - c_colors ;invert line colors for pstscrp
;;;;;;;;;;;;;;; extraploration contour ;;;;;;;;;;;;;;;;;;;;;;;;;;;;
if keyword_set(extrap) then  begin
    b1=[b(0,*),b,b(sz(1)-1,*)]  ;extrapolation in x-direction
    xx1=[2*xx(0)-xx(1),xx,2*xx(sz(1)-1)-xx(sz(1)-2)] ;extrap. x-axis
    b2=[[b1(*,0)],[b1],[b1(*,sz(1)-1)]] ;extrapolation in   y-direction
    yy1=[2*yy(0)-yy(1),yy,2*yy(sz(2)-1)-yy(sz(2)-2)] ;extrap.y-axis
endif else begin
    b2=b & xx1=xx & yy1=yy
endelse
;;;;;;;;;;;;;;; extraploration contour ;;;;;;;;;;;;;;;;;;;;;;;;;;;;
if keyword_set(extrap) then  begin
    e1=[e(0,*),e,e(sz(1)-1,*)]  ;extrapolation in x-direction
    xx1=[2*xx(0)-xx(1),xx,2*xx(sz(1)-1)-xx(sz(1)-2)] ;extrap. x-axis
    e2=[[e1(*,0)],[e1],[e1(*,sz(1)-1)]] ;extrapolation in   y-direction
    yy1=[2*yy(0)-yy(1),yy,2*yy(sz(2)-1)-yy(sz(2)-2)] ;extrap.y-axis
endif else begin
    e2=e & xx1=xx & yy1=yy
endelse
;;;;;;;;;;;;;;;; plot cotour plot ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
if n_elements(c_kwd) eq 0 then begin
    c_kwd=' ' 
endif else begin
    c_kwd=',' + c_kwd
endelse
c_kwd = c_kwd + ",c_colors = c_colors"
if n_elements(c_charsize) ne 0 then $
  c_kwd = c_kwd + ",c_charsize = c_charsize"
if n_elements(levels) ne 0 then $
  c_kwd = c_kwd + ",levels = levels"
if n_elements(nlevels) ne 0 then $
  c_kwd = c_kwd + ",nlevels = nlevels"
if keyword_set(follow) ne 0 then $
  c_kwd = c_kwd + ",/follow"
;;;;;;;;;;;;;;;; plot cotour plot ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
if n_elements(c_kwd2) eq 0 then begin
    c2_kwd=' ' 
endif else begin
    c2_kwd=',' + c2_kwd
endelse
c2_kwd = c2_kwd + ",c_colors = c2_colors"
if n_elements(c_charsize) ne 0 then $
  c2_kwd = c2_kwd + ",c_charsize = c_charsize"
if n_elements(l2evels) ne 0 then $
  c2_kwd = c2_kwd + ",levels = l2evels"
if n_elements(nlevels) ne 0 then $
  c2_kwd = c2_kwd + ",nlevels = nlevels"
if keyword_set(follow) ne 0 then $
  c2_kwd = c2_kwd + ",/follow"

r=execute("contour,b2,xx1,yy1,/noerase,xstyle=5,ystyle=5,"+ $ ;Do the contour
;        pos = [px(0),py(0), px(0)+swx,py(0)+swy],/dev,$
           "/overplot" + c_kwd)
r=execute("contour,e2,xx1,yy1,/noerase,xstyle=5,ystyle=5,"+ $ ;Do the contour
;        pos = [px(0),py(0), px(0)+swx,py(0)+swy],/dev,$
           "/overplot" + c2_kwd)
return
end


