 PRO VFIELD,U,V,X,Y, Noerase = noerase, Position = position, $
       Xstyle=xstyle,Ystyle=ystyle, Device = device, V_Color = v_color, $
       F_Color=f_color,Limit=limit, Scale=scale,Iskip = iskip, Jskip = jskip,$
       Margx=margx, Margy=margy,Title=title,Xtitle=xtitle,Ytitle=ytitle,$
	 I0skip=i0skip, J0skip=j0skip,SAMPLE=sample,$
       xticks=xticks,yticks=yticks,xtickv=xtickv,ytickv=ytickv,$
	xtickname=xtickname,ytickname=ytickname,xminor=xminor,yminor=yminor
;+
; NAME:
;    VFIELD
; PURPOSE:
;    vector plot
; CATEGORY:
;    General graphics.
; CALLING SEQUENCE:
;    VFIELD, U, V
; INPUTS:
;    U = 2 dimensional array to display.
;    V = 2 dimensional array to display.
; OPTIONAL INPUT PARAMETERS:
;    X = 1 dimensional array to display.
;    Y = 1 dimensional array to display.
; KEYWORD PARAMETERS:
;    SCALE = set scaling factor for the arrows
;    LIMIT = set limit for the minimum arrow size
;    ISKIP = set skip of arrow in x-direction
;    JSKIP = set skip of arrow in y-direction
;    I0SKIP = set skip of starting point of arrow in x-direction
;    J0SKIP = set skip of starting point of arrow in y-direction
;    MARGX = set margin of both x-side /data
;    MARGY = set margin of both y-side /data
;    POSITION = set coordinate of plotting flame, that given 4 element vector,
;		such that [x0,y0,x1,y1], where (x0,y0) and (y1,y1) are 
;		respectively the coordinate of lower left side and 
;		upper right side of the flame in normalized unit.
;		You can position in device unit by set keyword /dev.
;    TITLE = title given as strings
;    XTITLE = xtitle
;    YTITLE = ytitle
;    V_COLOR = The color index used to draw arrows
;    F_COLOR = The color index used to draw flame
;    SAMPLE = set to draw sample arrow
;    XTICKS = Same as the keyword of 'contour'
;    YTICKS = Same as the keyword of 'contour'
;    XTICKV = Same as the keyword of 'contour'
;    YTICKV = Same as the keyword of 'contour'
;    XTICKNAME = Same as the keyword of 'contour'
;    YTICKNAME = Same as the keyword of 'contour'
;    XMINOR = Same as the keyword of 'contour'
;    YMINOR = Same as the keyword of 'contour'
; OUTPUTS:
;    No explicit outputs.
; COMMON BLOCKS:
;    none.
; SIDE EFFECTS:
;    The currently selected display is affected.
; RESTRICTIONS:
;    None that are obvious.
; PROCEDURE:
;    If the device has scalable pixels then the image is written over
;    the plot window.
; MODIFICATION HISTORY:
;    DMS, May, 1988.
;    Yokoyama, ?????
;    Yngwie J. Matsumoto 1993.8.*
;    Yngwie J. Matsumoto 1993.10.8
;    matsu@a.phys.nagoya-u.ac.jp Jun 3 1995
;    matsu@a.phys.nagoya-u.ac.jp  Sat Feb 24 22:53:11 JST 1996
;-

;
;;;;;;;;; Error checking ;;;;;;;;;;;;;;;
        on_error,2                      ;Return to caller if an error occurs
        s = size(u)
        t = size(v)
        if s(0) ne 2 then begin
baduv:   message, 'U and V parameters must be 2D and same size.'
               endif
        if total(abs(s(0:2)-t(0:2))) ne 0 then goto,baduv
;
        if n_params(0) lt 3 then x = findgen(s(1)) else $
                if n_elements(x) ne s(1) then begin
badxy:                  message, 'X and Y arrays have incorrect size.'
                        endif
        if n_params(1) lt 4 then y = findgen(s(2)) else $
                if n_elements(y) ne s(2) then goto,badxy
;

        x0 = min(x)                     ;get scaling
        x1 = max(x)
        y0 = min(y)
        y1 = max(y)
        dxp= (x1-x0)/(s(1)-1.0)		;unit data/element (= dx/d_element)
        dyp= (y1-y0)/(s(2)-1.0) 
;;;;;;;;; default parameters ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; defaults value of iskip and jskip;;;
if n_elements(iskip)*n_elements(jskip) eq 0 then begin
if n_elements(position) eq 0 then begin
	px = !x.window * !d.x_vsize	;Get size of window in device units
	py = !y.window * !d.y_vsize
	py = py-8.			;offset in y-direction(in devece unit)
	swx = px(1)-px(0)		;Size in x in device units
	swy = py(1)-py(0)		;Size in Y
endif else begin
	swx=position(2)-position(0)
      swy=position(3)-position(1)
endelse
	len_arr_x=swx/20.   
	len_arr_y=swy/20.
	len_arr=len_arr_y > len_arr_x    ;arrow's length in device unit
	iskip=fix(s(1)*len_arr/swx)
	jskip=fix(s(2)*len_arr/swy)
      if iskip eq 0 then iskip=1
      if jskip eq 0 then jskip=1
print,'iskip=',iskip
print,'jskip=',jskip
endif

if n_elements(i0skip) eq 0 then i0skip=0
if n_elements(j0skip) eq 0 then j0skip=0
if n_elements(limit) eq 0 then limit=0.0
if n_elements(xstyle) eq 0 then xstyle=1
if n_elements(ystyle) eq 0 then ystyle=1
if n_elements(xticks) eq 0 then xticks=!x.ticks
if n_elements(yticks) eq 0 then yticks=!y.ticks
if n_elements(xtickv) eq 0 then xtickv=!x.tickv
if n_elements(ytickv) eq 0 then ytickv=!y.tickv
if n_elements(xminor) eq 0 then xminor=!x.minor
if n_elements(yminor) eq 0 then yminor=!y.minor
if n_elements(xtickname) eq 0 then xtickname=''
if n_elements(ytickname) eq 0 then ytickname=''
if n_elements(margx) eq 0 then margx=dxp*iskip
if n_elements(margy) eq 0 then margy=dyp*jskip
scalex=max(abs(u))/dxp/iskip
scaley=max(abs(v))/dyp/jskip
if n_elements(scale) eq 0 then scale = scalex > scaley
print,'velocity scale=',scale
if n_elements(title) le 0 then title = ''
if n_elements(xtitle) le 0 then xtitle=''
if n_elements(ytitle) le 0 then ytitle=''
if n_elements(f_color) eq 0 then f_color = !p.color
if n_elements(v_color) eq 0 then v_color = !p.color
;;;;;;;;; Flame ;;;;;;;;;;;;;;;
        if n_elements(position) eq 0 then begin
            plot,[x0-margx,x1+margx],[y1+margy,y0-margy],/nodata,$
            xstyle=xstyle,ystyle=xstyle, $
            title=title, xtitle=xtitle,ytitle=ytitle,$
              noerase=noerase, color=f_color,$
            xticks=xticks,yticks=yticks,xtickv=xtickv,ytickv=ytickv,$
            xtickname=xtickname,ytickname=ytickname,$
            xminor=xminor,yminor=yminor
        endif else begin
           plot,[x0-margx,x1+margx],[y1+margy,y0-margy],/nodata,$
              xstyle=xstyle,ystyle=xstyle, $
              title=title,xtitle=xtitle,ytitle=ytitle, $
              xticks=xticks,yticks=yticks,xtickv=xtickv,ytickv=ytickv,$
              xtickname=xtickname,ytickname=ytickname,$
              xminor=xminor,yminor=yminor,$
              noerase=noerase, color=f_color, position=position,device=device
        endelse

px = !x.window * !d.x_vsize	;Get size of window in device units
py = !y.window * !d.y_vsize
py = py-8.
swx = px(1)-px(0)		;Size in x in device units
swy = py(1)-py(0)		;Size in Y

        ori_dev=convert_coord(0.,0.,/data,/to_dev)
					; coordinate of origin in dev. unit
        r = .4                          ;len of arrow head
        angle = 22.5 * !dtor            ;Angle of arrowhead
        st = r * sin(angle)            ;sin 22.5 degs * length of head
        ct = r * cos(angle)

        for i=i0skip,s(1)-1,iskip do begin
           xp=x0+float(i)*dxp		;λ(xp,yp) in data unit
        for j=j0skip,s(2)-1,jskip do begin
           yp=y0+float(j)*dyp

           vecx=u(i,j)
           vecy=v(i,j)
           vec=sqrt(vecx*vecx+vecy*vecy)	;Ĺ in data unit

           if vec ge limit then begin
              vxp=vecx/scale			;Ĺ scaling
              vyp=vecy/scale
              xp1=xp+vxp			;ν in data unit
              yp1=yp+vyp
              vec_dev=convert_coord(vecx,vecy,/data,/to_dev)-ori_dev
              xpa_dev=ct*vec_dev(0)-st*vec_dev(1)
              ypa_dev=ct*vec_dev(1)+st*vec_dev(0)
              xpb_dev=ct*vec_dev(0)+st*vec_dev(1)
              ypb_dev=ct*vec_dev(1)-st*vec_dev(0)
              pa=convert_coord(xpa_dev+ori_dev(0), $
                               ypa_dev+ori_dev(1),/dev,/to_data)/scale 
              pb=convert_coord(xpb_dev+ori_dev(0),$
                               ypb_dev+ori_dev(1),/dev,/to_data)/scale
              plots,[xp,xp1,xp1-pa(0),xp1,xp1-pb(0)],$
                    [yp,yp1,yp1-pa(1),yp1,yp1-pb(1)],$
                    color=v_color
           endif
        endfor
        endfor
;;;;;;;;;;;; draw sample arrow
	if keyword_set(sample) then begin
              ;;  ʸФ륪եå
              charsize=!p.charsize
              if(charsize eq 0.0) then charsize=1.0
              ;; y-offset of sample arrow in device unit
              dy_dev=charsize*!d.y_ch_size*0.5 
              ;; offset  device unit  data unit Ѵ
              sample_offset =convert_coord(ori_dev(0), $
                              dy_dev+ori_dev(1),/dev,/to_data)
              xp=x0+0.5*(x1-x0)
              yp=y0+(y1-y0)+ abs(y(1)-y(0))*0.5
              vecx=float(sample)    ;; ץĹ in data unit
              vecy=0.0
              vxp=vecx/scale
              vyp=vecy/scale
              xp1=xp+vxp
              yp1=yp+vyp
              vec_dev=convert_coord(vecx,vecy,/data,/to_dev)-ori_dev
              xpa_dev=ct*vec_dev(0)-st*vec_dev(1)
              ypa_dev=ct*vec_dev(1)+st*vec_dev(0) 
              xpb_dev=ct*vec_dev(0)+st*vec_dev(1)
              ypb_dev=ct*vec_dev(1)-st*vec_dev(0) 
              pa=convert_coord(xpa_dev+ori_dev(0), $
                               ypa_dev+ori_dev(1),/dev,/to_data)/scale 
              pb=convert_coord(xpb_dev+ori_dev(0),$
                               ypb_dev+ori_dev(1),/dev,/to_data)/scale
              plots,[xp,xp1,xp1-pa(0),xp1,xp1-pb(0)] + sample_offset(0),$
                    [yp,yp1,yp1-pa(1),yp1,yp1-pb(1)] + sample_offset(1),$
                    color=v_color
;              xp=px(0)+0.85*swx	;lavel (xp,yp) in dev.unit
;              yp=py(0)+1.03*swy
;              xyouts,xp,yp,/dev,' = '+ STRING(FORMAT='(F5.2)',sample)
             xyouts,xp1,yp+sample_offset(1)*0.2 ,$
               ' = '+ STRING(FORMAT='(F5.2)',sample),$
               size=!p.charsize
	endif

return
end



