;+
; write2dss_png.pro (2004/09/04)
;    read data files generated by 2D HD code automatically, 
;    create png figure files.
;- 
;
; explanation for parameters (2002/06/28/ kana)
;   phys: parameter for color scale
;	t_max: maximum value for color scale
;	t_min: minimum value for color scale
;   physc: parameter for contour
;	c_lev, c2_lev: contour levels
;   sv1, sv2: parameters for arrows(sv1: x-comp., sv2: y-comp.)
;	j_skip, k_skip: the number of mesh points skipped in x-, y-direction 
;	v_scale: scale of arrows
;
;;; for png output
xsize=640 & ysize=640
set_plot,'Z'
device,set_resolution=[xsize,ysize]
;
!p.font=1
!p.charsize=2.5
!x.thick=2
!y.thick=2
!z.thick=2
!p.thick=1.5
;
print,'Input "v" or "b"'
read,' for vector.:',parameter2
if (parameter2 eq "") then parameter2='v'
;parameter2='v'
;parameter2='b'
;
fn1=findfile('sample*.d')
fn=strmid(fn1,0,7)
;
its=0
ite=size(fn,/n_elements)-1
;
for l=its,ite do begin
i=0
erase
;
ctitle='title'
precision='FLOAT'
timei=0.
;
OPENR, Unit1, fn(l)+'.d', /GET_LUN
READF, Unit1, ctitle, FORMAT='(A11)'
READF, Unit1, ixs, ixe, izs, ize, FORMAT='(4I5)'
READF, Unit1, timei, FORMAT='(/,/,/,/,7X,F15.7)'
READF, Unit1, precision, FORMAT='(A6)'
FREE_LUN, Unit1
;
case precision of
'FLOAT ': begin
;print,'data type: ',precision
x = FLTARR(ixe-ixs+1)
y = FLTARR(iye-iys+1)
rho = FLTARR(ixe-ixs+1, iye-iys+1)
vx = FLTARR(ixe-ixs+1, iye-iys+1)
vy = FLTARR(ixe-ixs+1, iye-iys+1)
bx = FLTARR(ixe-ixs+1, iye-iys+1)
by = FLTARR(ixe-ixs+1, iye-iys+1)
end
else: begin
;print,'data type: ',precision
x = DBLARR(ixe-ixs+1)
y = DBLARR(iye-iys+1)
rho = DBLARR(ixe-ixs+1, iye-iys+1)
vx = DBLARR(ixe-ixs+1, iye-iys+1)
vy = DBLARR(ixe-ixs+1, iye-iys+1)
bx = DBLARR(ixe-ixs+1, iye-iys+1)
by = DBLARR(ixe-ixs+1, iye-iys+1)
end
endcase
;
OPENR, Unit2, fn(l)+'.D', /F77_UNFORMATTED, /GET_LUN, /SWAP_IF_LITTLE_ENDIAN
READU, Unit2, x
READU, Unit2, y
READU, Unit2, rho
READU, Unit2, vx
READU, Unit2, vy
READU, Unit2, bx
READU, Unit2, by
FREE_LUN, Unit2
;
print,'t=',timei
;
phys=rho
physc=rho
;
; parameters for tv
t_max=1.0
t_min=0.
;t_max=max(phys)
;t_min=min(phys)
;
; parameters for contour
;c_top=!d.n_colors
c_top=!d.table_size
c_lev=findgen(10)*0.05+0.5
c2_lev=findgen(10)*0.05
;
; parameters for vector
i_skip=ixe/16
j_skip=iye/16
;
; parameters for vector and axis
    print,'x-y slice'
    sphyst=phys
    sphysc=physc
    xax=x
    yax=y
    xtm='!16x!3'
    ytm='!16y!3' 
    case parameter2 of 
        'v':begin
	    sv1=vx
	    sv2=vy
            end
        'b':begin
            sv1=bx
            sv2=by
            end
    endcase
;
;v_scale=long(max(sqrt(sv1^2+sv2^2)))/((x(1)-x(0))*i_skip)
;if (v_scale le 0.) then $
;  v_scale=max(sqrt(sv1^2+sv2^2))/((x(1)-x(0))*i_skip) 
v_scale=56.
;
beginp,[0.15,0.15,0.85,0.85]
tvcn2ve,$
bytscl(phys,top=c_top-3,max=t_max,min=t_min)+1,$
physc,sv1,sv2,physc,xax,yax,/asp,xtitle=xtm,ytitle=ytm,$
;iskip=i_skip,jskip=j_skip,v_col=!d.n_colors,scale=v_scale,$
iskip=i_skip,jskip=j_skip,v_col=-1,scale=v_scale,$
levels=c_lev,c_col=1,$
l2evels=c2_lev,c2_col=0
endp
;
; parameters for title
stime=STRING(FORMAT='(F5.2)',timei)
xyouts,-max(x)*0.2,max(y)*1.05,/data,'!8t!3 ='+stime
;
; the size of unit arrow
v0=5.0
arrow,max(x)*0.6,max(y)*1.10,max(x)*0.6+(v0/v_scale),max(y)*1.10,/data,$
hsize=30.*v0/v_scale
xyouts,max(x)*0.65+(v0/v_scale),max(y)*1.09,'!8v=1.0 v!D0!N!3',/data
;
beginp,[0.85,0.15,0.87,0.85]
c_scale,sphyst,top=c_top-3+1,max=t_max,min=t_min,bottom=1,/portrait,yaxis=1
endp
;
tvlct,r,g,b,/get
write_png,'sample'+strmid(fn(l),6,1)+'.png', TVRD(),r,g,b
endfor
set_plot,'X'
;
end